%
%   Fast Fourier Transform (FFT)
%
%   BENESCO Lecture series on signal analysis
%   Mojtaba Bandarabadi
%   Inselspital, Bern, 18 November 2016
%   for any question send email to: m.bandarabadi@gmail.com
%


%% clean up!
clear; close all; clc


%% let's generate an artificial signal with three frequency components
sampFreq = 256;                          % sampling frequency of signal
lenSig = 8*sampFreq;                    % length of signal in sample (8s)
time = 0:1/sampFreq:(lenSig-1)/sampFreq; % define time axis in second

comp1 = 1.0*cos(2*pi*8*time+pi);         % 8 Hz with pi phase
comp2 = 0.5*cos(2*pi*20*time+pi/2);      % 20 Hz with +pi/2 phase
comp3 = 0.2*cos(2*pi*80*time-pi/2);      % 80 Hz with -pi/2 phase
signal = comp1 + comp2 + comp3;


% and then plot the signals
figure('Position',[80,80,800,600],'Color',[1 1 1]);
subplot(3,2,[1,2]); hold on;             % plot 1 second of components
plot(time(1:sampFreq),comp1(1:sampFreq),'linewidth',1.5);
plot(time(1:sampFreq),comp2(1:sampFreq),'linewidth',1.5);
plot(time(1:sampFreq),comp3(1:sampFreq),'linewidth',1.5);
box off
title('Components of signal')
xlabel('Time (sec)')
ylabel('Amplitude')
set(gca,'fontsize',12);

subplot(3,2,[3,4]);                      % plot 1 second of signal
plot(time(1:sampFreq),signal(1:sampFreq),'linewidth',2);
box off
title('Signal')
xlabel('Time (sec)')
ylabel('Amplitude')
set(gca,'fontsize',12);


%% Discrete Fourier Transform (DFT)
% DFT decomposes signal into its frequency components
% FFT is an implementation of DFT

% and now Fast Fourier Transform (FFT)
fftSig = fft(signal);                 % FFT
fftSig = fftSig(1:lenSig/2+1);        % take single-sided spectrum
freq = 0:sampFreq/lenSig:sampFreq/2;  % corresponding frequencies

absFFT = abs(fftSig/lenSig);          % absolute values
absFFT(2:end-1) = 2*absFFT(2:end-1);  % multiply single-sided spectrum by 2

phaseFFT = angle(fftSig);             % phases of frequencies


% and then plot FFT output
% plot absolute values of FFT
subplot(3,2,5);
plot(freq,absFFT,'linewidth',2)
box off
title('Amplitude spectrum')
xlabel('Frequency (Hz)')
ylabel('|fft(f)|')
set(gca,'fontsize',12);
axis tight


% plot phases of components
subplot(3,2,6);
idxComp = [find(freq==8),find(freq==20),...
    find(freq==80)]; % index of main frequency components (8,20,and 80 Hz)
bar(freq(idxComp),phaseFFT(idxComp))
box off
title('Phase of frequency components')
xlabel('Frequency (Hz)')
ylabel('Phase')
set(gca,'fontsize',12);
xlim([freq(1) freq(end)])
yticks([-pi -pi/2 0 pi/2 pi])
yticklabels({'-pi','-pi/2','0','pi/2','pi'})
ylim([-pi pi])

