%
%   Power Spectral Density (PSD)
%
%   BENESCO Lecture series on signal analysis
%   Mojtaba Bandarabadi
%   Inselspital, Bern, 18 November 2016
%   for any question send email to: m.bandarabadi@gmail.com
%


%% clean up!
clear; close all; clc


%% let's generate an artificial signal with three frequency components
sampFreq = 256;                          % sampling frequency of signal
lenSig = 8*sampFreq;                    % length of signal in sample (8s)
time = 0:1/sampFreq:(lenSig-1)/sampFreq; % define time axis in second

comp1 = 1.0*cos(2*pi*8*time+pi);         % 8 Hz with pi phase
comp2 = 0.5*cos(2*pi*20*time+pi/2);      % 20 Hz with +pi/2 phase
comp3 = 0.2*cos(2*pi*80*time-pi/2);      % 80 Hz with -pi/2 phase
signal = comp1 + comp2 + comp3;

% and then plot the signals
figure('Position',[80,80,800,400],'Color',[1 1 1]);
subplot(2,2,[1,2]);                      % plot 1 second of signal
plot(time(1:sampFreq),signal(1:sampFreq),'linewidth',2);
box off
title('Signal')
xlabel('Time (sec)')
ylabel('amplitude')
set(gca,'fontsize',12);


%% Power Spectral Density (PSD)
% There are many ways to estimate PSD in MATLAB

% Estimate PSD from FFT
fftSig = fft(signal);                 % FFT
fftSig = fftSig(1:lenSig/2+1);        % take single-sided spectrum
psdFFT = (1/(sampFreq*lenSig)) * abs(fftSig).^2;
psdFFT(2:end-1) = 2*psdFFT(2:end-1);
freq = 0:sampFreq/lenSig:sampFreq/2;  % corresponding frequencies

subplot(2,2,3);
plot(freq,psdFFT,'linewidth',1.5)
grid on
title('PSD using FFT')
xlabel('Frequency (Hz)')
ylabel('Power/Freq.')
set(gca,'fontsize',12);
xlim([freq(1) freq(end)])

subplot(2,2,4);
plot(freq,10*log10(psdFFT),'linewidth',1.5)
grid on
title('PSD using FFT')
xlabel('Frequency (Hz)')
ylabel('Power/Freq. (dB/Hz)')
set(gca,'fontsize',12);
xlim([freq(1) freq(end)])
ylim([min(10*log10(psdFFT))-10 0])

