%
%   Spectral leakage
%
%   BENESCO Lecture series on signal analysis
%   Mojtaba Bandarabadi
%   Inselspital, Bern, 18 November 2016
%   for any question send email to: m.bandarabadi@gmail.com
%


%% clean up!
clear; close all; clc


%% One frequency - No windowing
sampFreq = 256;                          % sampling frequency of signal
lenSig = 4*sampFreq;                     % length of signal in sample
time = 0:1/sampFreq:(lenSig-1)/sampFreq; % define time axis in second
signal1 = 1.0*cos(2*pi*8*time+pi);       % 8 Hz with pi phase
signal2 = 1.0*cos(2*pi*8.21*time+pi);    % 8.21 Hz with pi phase

% plot the signals
figure('Position',[80,80,800,400],'Color',[1 1 1]);
subplot(2,2,[1,2]);                      % plot 1 second of signal
plot(time(1:lenSig),signal1(1:lenSig),'linewidth',2); hold on
plot(time(1:lenSig),signal2(1:lenSig),'r','linewidth',2); hold off
box off
title('Signals')
xlabel('Time (sec)')
ylabel('Amplitude')
set(gca,'fontsize',12);
legend('signal 1', 'signal 2')
xlim([0 1])

% PSD using periodogram function with rectangular window
window = rectwin(length(signal1));
[psd1,freq] = periodogram(signal1,window,2*sampFreq,sampFreq);
subplot(2,2,3);
plot(freq,10*log10(psd1),'linewidth',2)
grid on
title('PSD of signal 1')
xlabel('Frequency (Hz)')
ylabel('Power/Freq. (dB/Hz)')
set(gca,'fontsize',12);
xlim([freq(1) freq(end)])
ylim([min(10*log10(psd1))-10 0])

[psd2,freq] = periodogram(signal2,window,2*sampFreq,sampFreq);
subplot(2,2,4);
plot(freq,10*log10(psd2),'r','linewidth',2); hold on
grid on
title('PSD of signal 2')
xlabel('Frequency (Hz)')
ylabel('Power/Freq. (dB/Hz)')
set(gca,'fontsize',12);
xlim([freq(1) freq(end)])
ylim([min(10*log10(psd1))-10 0])



%% One frequency - Windowing
sampFreq = 256;                          % sampling frequency of signal
lenSig = 4*sampFreq;                     % length of signal in sample
time = 0:1/sampFreq:(lenSig-1)/sampFreq; % define time axis in second
signal1 = 1.0*cos(2*pi*8*time+pi);      % 8 Hz with pi phase
signal2 = 1.0*cos(2*pi*8.21*time+pi);   % 8.21 Hz with pi phase

% plot the signals
figure('Position',[80,80,800,600],'Color',[1 1 1]);
subplot(3,2,[1,2]);                     % plot 1 second of signal
plot(time(1:lenSig),signal1(1:lenSig),'linewidth',2); hold on
plot(time(1:lenSig),signal2(1:lenSig),'r','linewidth',2); hold off
box off
title('Signals')
xlabel('Time (sec)')
ylabel('Amplitude')
set(gca,'fontsize',12);
legend('signal 1', 'signal 2')
xlim([0 1])

subplot(3,2,[3,4]);                     % plot 1 second of windowed signal
window = hann(sampFreq);
plot(time(1:sampFreq),window'.*signal1(1:sampFreq),'linewidth',2); hold on
plot(time(1:sampFreq),window'.*signal2(1:sampFreq),'r','linewidth',2); hold off
box off
title('Windowed signals')
xlabel('Time (sec)')
ylabel('Amplitude')
set(gca,'fontsize',12);
legend('signal 1', 'signal 2')
xlim([0 1])


% PSD using periodogram function with rectangular window
window = rectwin(length(signal1));
[psd1rect,freq] = periodogram(signal1,window,2*sampFreq,sampFreq);
subplot(3,2,5);
plot(freq,10*log10(psd1rect),'b:','linewidth',2); hold on
grid on
title('PSD of signal 1')
xlabel('Frequency (Hz)')
ylabel('Power/Freq. (dB/Hz)')
set(gca,'fontsize',12);
xlim([freq(1) freq(end)])
ylim([min(10*log10(psd1rect))-10 0])

[psd2,freq] = periodogram(signal2,window,2*sampFreq,sampFreq);
subplot(3,2,6);
plot(freq,10*log10(psd2),'r:','linewidth',2); hold on
grid on
title('PSD of signal 2')
xlabel('Frequency (Hz)')
ylabel('Power/Freq. (dB/Hz)')
set(gca,'fontsize',12);
xlim([freq(1) freq(end)])
ylim([min(10*log10(psd1rect))-10 0])


% PSD using periodogram function with hanning window
window = hann(length(signal1));
[psd1,freq] = periodogram(signal1,window,2*sampFreq,sampFreq);
subplot(3,2,5);
plot(freq,10*log10(psd1),'linewidth',2)
grid on
title('PSD of signal 1')
xlabel('Frequency (Hz)')
ylabel('Power/Freq. (dB/Hz)')
set(gca,'fontsize',12);
xlim([freq(1) freq(end)])
ylim([min(10*log10(psd1rect))-10 0])

[psd2,freq] = periodogram(signal2,window,2*sampFreq,sampFreq);
subplot(3,2,6);
plot(freq,10*log10(psd2),'r','linewidth',2); hold on
grid on
title('PSD of signal 2')
xlabel('Frequency (Hz)')
ylabel('Power/Freq. (dB/Hz)')
set(gca,'fontsize',12);
xlim([freq(1) freq(end)])
ylim([min(10*log10(psd1rect))-10 0])



%% Several frequency
% let's generate an artificial signal with three frequency components
sampFreq = 256;                          % sampling frequency of signal
lenSig = 2*sampFreq;                     % length of signal in sample
time = 0:1/sampFreq:(lenSig-1)/sampFreq; % define time axis in second
comp1 = 1.0*cos(2*pi*8.21*time+pi);      % 8.21 Hz with pi phase
comp2 = 0.5*cos(2*pi*20.23*time+pi/2);   % 20.23 Hz with +pi/2 phase
comp3 = 0.2*cos(2*pi*80*time-pi/2);      % 80 Hz with -pi/2 phase
signal = comp1 + comp2 + comp3;

% and then plot the signals
figure('Position',[80,80,800,600],'Color',[1 1 1]);
subplot(3,2,[1,2]); hold on;             % plot 1 second of components
plot(time(1:lenSig),comp1(1:lenSig),'linewidth',1.5);
plot(time(1:lenSig),comp2(1:lenSig),'linewidth',1.5);
plot(time(1:lenSig),comp3(1:lenSig),'linewidth',1.5);
box off
title('Components of signal')
xlabel('Time (sec)')
ylabel('Amplitude')
set(gca,'fontsize',12);
xlim([0 2])

subplot(3,2,[3,4]);                      % plot 1 second of signal
plot(time(1:lenSig),signal(1:lenSig),'linewidth',2);
box off
title('Signal')
xlabel('Time (sec)')
ylabel('Amplitude')
set(gca,'fontsize',12);
xlim([0 2])


% PSD using periodogram function with hanning window
window = hann(length(signal));
[psdWindow,freq] = periodogram(signal,window,sampFreq,sampFreq);
subplot(3,2,6);
plot(freq,10*log10(psdWindow),'linewidth',2)
grid on
title('PSD using hanning window')
xlabel('frequency (Hz)')
ylabel('Power/Freq. (dB/Hz)')
set(gca,'fontsize',12);
xlim([freq(1) freq(end)])
ylim([min(10*log10(psdWindow))-10 0])


% PSD using periodogram function with rectangular window
window = rectwin(length(signal));
[psdRect,freq] = periodogram(signal,window,sampFreq,sampFreq,'psd');
subplot(3,2,5);
plot(freq,10*log10(psdRect),'linewidth',2)
grid on
title('PSD without windowing (rectangular)')
xlabel('frequency (Hz)')
ylabel('Power/Freq. (dB/Hz)')
set(gca,'fontsize',12);
xlim([freq(1) freq(end)])
ylim([min(10*log10(psdWindow))-10 0])
