%
%   Power Spectral Density (PSD)
%
%   BENESCO Lecture series on signal analysis
%   Mojtaba Bandarabadi
%   Inselspital, Bern, 18 November 2016
%   for any question send email to: m.bandarabadi@gmail.com
%


%% clean up!
clear; close all; clc


%% Load data
% provided 'sampleData.mat' file contains LFP recordings from mouse
% rawSig: 25-sec LFP recordings from mouse, contains Wake NREM REM stages
% fs: sampling frequency of LFP signal, fs = 1000 Hz.
load('sampleData')

figure('Position',[80,80,800,600],'Color',[1 1 1]);
t = 0:1/fs:(length(rawSig)-1)/fs;   % define time axis
subplot(211); plot(t,rawSig);       % plot signal
box off
xlabel('Time (s)')    % x label
ylabel('Amplitude')   % y label
set(gca,'fontsize',12);


%% Estimation using Welch's method
window = 4*fs; % 4-sec windows for minimum frequency content of 0.25 Hz
overlap = round(0.5*window); % 50 percent overlap
[pxx,f] = pwelch(rawSig,window,overlap,fs,fs); % estimate PSD using Welch
subplot(212); hold on
plot(f,10*log10(pxx),'linewidth',2); % plot PSD in db
box off
xlim([0 50]) % limit frequency axis to [0 80] Hz in plot
xlabel('Frequency (Hz)')
ylabel('Magnitude (dB)')
set(gca,'fontsize',12);


%% Estimation using periodogram
window = rectwin(length(rawSig));
[pxx1,f1] = periodogram(rawSig,window,fs,fs);
plot(f1,10*log10(pxx1),'linewidth',2);
xlim([0 80])
xlabel('Frequency (Hz)')
ylabel('Magnitude (dB)')
legend({'Welch method','Periodogram'})
set(gca,'fontsize',12);
hold off
