%
%   Time-frequency representation using short-time Fourier transform (STFT)
%
%   BENESCO Lecture series on signal analysis
%   Mojtaba Bandarabadi
%   Inselspital, Bern, 18 November 2016
%   for any question send email to: m.bandarabadi@gmail.com
%


%% clean up
clear; close all; clc


%% Load sample data
% rawSig: 25-sec LFP recordings from mouse, contains Wake NREM REM stages
% fs: sampling frequency of LFP signal, fs = 1000 Hz.
load('sampleData')


%% Time-frequency representation using STFT
minfreq = 0.5;
maxfreq = 20;

window = 1*fs;                % window size in sample
overlap = round(0.96*window); % overlap is an integer value, less than window

[SC,freq,time] = spectrogram(rawSig,window,overlap,2*fs,fs);
SC = abs(SC(freq>=minfreq&freq<=maxfreq,:));
freq = freq(freq>=minfreq&freq<=maxfreq);
args = {time,freq,SC.^2};

figure('Color',[1 1 1]);
surf(args{:},'edgecolor','none');
view(0,90);
axis tight;
shading interp;
colormap(parula(128));
h = colorbar;
h.Label.String = 'Power';
xlabel('Time (s)');
ylabel('frequency (Hz)');

