%
%   Find pseudo-frequencies corresponding to the scales of wavelet
%   transform
%
%   BENESCO Lecture series on signal analysis
%   Mojtaba Bandarabadi
%   Inselspital, Bern, 18 November 2016
%   for any question send email to: m.bandarabadi@gmail.com
%


%% clean up
clear; close all; clc


%% scale to frequency
%  Here is an example from MATLAB help files, to show how to match a scale
%  of CWT into an approximate frequency.
Fs = 1000; % sampling frequency of signal
t = 0:1/Fs:1-1/Fs; % time axis
x = 1.5*cos(2*pi*100*t).*(t<0.25)+1.5*cos(2*pi*50*t).*(t>0.5 & t<=0.75);
x = x+0.05*randn(size(t)); % add noise to signal

figure('Color',[1 1 1]);
subplot(211); plot(t, x)
xlabel('Time');
ylabel('Amplitude');
set(gca,'fontsize',12); box off

numvoices = 32;
a0 = 2^(1/numvoices);
scales = a0.^(2*numvoices:1/numvoices:6*numvoices); % scales
cfs = cwt(x,scales,'morl'); % apply cwt
pfreq = scal2frq(scales,'morl',1/Fs); % match scale to frequency
subplot(212);
contour(t,pfreq,abs(cfs).^2);
axis tight;
grid on;
xlabel('Time');
ylabel('Approximate Frequency (Hz)');
title('CWT with Time vs Frequency');
set(gca,'fontsize',12); box off
