%
%   Time-frequency representation using Continuous wavelet transform (CWT)
%
%   BENESCO Lecture series on signal analysis
%   Mojtaba Bandarabadi
%   Inselspital, Bern, 18 November 2016
%   for any question send email to: m.bandarabadi@gmail.com
%


%% clean up
clear; close all; clc


%% Load sample data
% rawSig: 25-sec LFP recordings from mouse, contains Wake NREM REM stages
% fs: sampling frequency of LFP signal, fs = 1000 Hz.
load('sampleData')


%% Time-frequency representation using CWT
minfreq = 0.5; % minimum frequency for CWT
maxfreq = 20;  % maximum frequency for CWT

dt = 1/fs;           % time between two samples (1/sampling freq.)
time = dt:dt:length(rawSig)/fs; % define time axis
f0 = 6/(2*pi);       % central frequency of wavelet function ('morl')
NumVoices = 32;
a0 = 2^(1/NumVoices);
minscale = f0/(maxfreq*dt);
maxscale = f0/(minfreq*dt);
minscale = floor(NumVoices*log2(minscale));
maxscale = ceil(NumVoices*log2(maxscale));
scales = a0.^(minscale:maxscale).*dt;
cwtx = cwtft({rawSig,dt},'wavelet','morl','scales',scales); % CWT
SC = abs(cwtx.cfs);
freq = cwtx.frequencies;
args = {time,freq,SC.^2};

figure('Color',[1 1 1]);
surf(time,cwtx.frequencies,abs(cwtx.cfs).^2,'edgecolor','none');
view(0,90);
axis tight;
shading interp;
colormap(parula(128));
h = colorbar;
h.Label.String = 'Power';
xlabel('Time (s)');
ylabel('Frequency (Hz)');


